@@ -37,6 +37,7 @@ extern "C" {
 typedef enum {
 CHUNKER_RESET,
 CHUNKER_NOCHUNKS,
+CHUNKER_KEYS,
 CHUNKER_ROWS,
 CHUNKER_COLUMNS,
 CHUNKER_RECURSIVE,
@@ -86,7 +87,9 @@ typedef struct GokChunk {
 } GokChunk;
 
 void gok_chunker_initialize (void);
-gboolean gok_chunker_chunk_all (gint TypeChunks, gint OrderChunks);
+gboolean gok_chunker_chunk_all (ChunkTypes TypeChunks, ChunkOrder chunkOrder);
+gboolean gok_chunker_chunk_none (void);
+gboolean gok_chunker_chunk_keys (gboolean TopBottom, gboolean LeftRight);
 gboolean gok_chunker_chunk_rows (gboolean TopBottom, gboolean LeftRight);
 gboolean gok_chunker_chunk_columns (gboolean TopBottom, gboolean LeftRight);
 void gok_chunker_init_chunk (GokChunk* pChunk);
@@ -133,6 +136,7 @@ void gok_chunker_state_next (void);
 void gok_chunker_state_jump (gchar* NameState); 
 gboolean gok_chunker_select_chunk (void);
 void gok_chunker_highlight_chunk (GokChunk* pChunk);
+void gok_chunker_highlight_chunk_with_key (GokKey* pKey);
 void gok_chunker_unhighlight_chunk (GokChunk* pChunk);
 void gok_chunker_highlight_center_key (void);
 void gok_chunker_highlight_first_chunk (void);
@@ -143,7 +147,7 @@ gboolean gok_chunker_insert_item_col_ttb (GokChunk* pChunk, GokChunkitem* pChunk
 gboolean gok_chunker_insert_item_col_btt (GokChunk* pChunk, GokChunkitem* pChunkitem);
 void gok_chunker_dump_chunks (void);
 void gok_chunker_dump_chunk (GokChunk* pChunk);
-GokChunk* gok_chunker_chunk_group (GokChunk* pChunk, gint NumberGroups, gint Order, gboolean bCanReorder);
+GokChunk* gok_chunker_chunk_group (GokChunk* pChunk, gint NumberGroups, ChunkOrder Order, gboolean bCanReorder);
 void gok_chunker_add_chunkitem (GokChunk* pChunk, GokChunkitem* pChunkitem);
 gint gok_chunker_count_chunkitems (GokChunk* pChunk);
 GokChunk* gok_chunker_make_2_vertical (GokChunk* pChunk);
