@@ -32,7 +32,8 @@
 #include "main.h"
 
 /* Private functions */
-void gok_data_set_name_accessmethod_internal (char* Name);
+static void gok_data_set_name_accessmethod_internal (const char* Name);
+static const GokComposeType gok_data_compose_type_from_string (const gchar *s);
 
 /* pointer to the first GokData setting */
 static GokSetting* m_pSettingFirst;
@@ -65,8 +66,10 @@ static gboolean m_bDriveCorePointer = FALSE;
 static gboolean m_bExpandWindow;
 static gboolean m_bUseXkbKbd;
 static gboolean m_bUseAuxDicts = FALSE;
+static gdouble m_ValuatorSensitivity = 0.25;
 static gchar *m_AuxDicts = NULL;
 static GokDockType m_eDockType;
+static GokComposeType compose_keyboard_type;
 
 /* This GConfClient will be shared by all gconf_client
  * calls in this file.
@@ -230,6 +233,9 @@ gboolean gok_data_read_settings (void)
 	gok_gconf_get_bool (gconf_client, GOK_GCONF_USE_AUX_DICTS, 
 			         &m_bUseAuxDicts);
 
+ 	gok_gconf_get_double (gconf_client, GOK_GCONF_VALUATOR_SENSITIVITY,
+                                 &m_ValuatorSensitivity);
+
         gok_gconf_get_string (gconf_client, GOK_GCONF_AUX_DICTS, 
 			         &m_AuxDicts);
 
@@ -238,6 +244,11 @@ gboolean gok_data_read_settings (void)
         m_eDockType = gok_data_dock_type_from_string (a_gchar);
 	g_free (a_gchar);
 
+	gok_gconf_get_string (gconf_client, GOK_GCONF_COMPOSE_KBD_TYPE,
+			      &a_gchar);
+        compose_keyboard_type = gok_data_compose_type_from_string (a_gchar);
+	g_free (a_gchar);
+
 	/* Read in settings for individual access methods */
 	access_method_dirs
 	    = gconf_client_all_dirs (gconf_client,
@@ -414,6 +425,36 @@ gboolean gok_data_get_control_values (GokControl* pControl)
 }
 */
 
+
+static void
+gok_data_fill_setting_values (gint settingValue, gchar *settingValueString, gint *Value, gchar **ValueString)
+{
+    /* set the given value to the new value */
+    if (settingValueString != NULL)
+    {
+	if (ValueString != NULL)
+	{
+	    *ValueString = settingValueString;
+	}
+	if (Value != NULL)
+	{
+	    *Value = 0;
+	}
+    }
+    else
+    {
+	if (Value != NULL)
+	{
+	    *Value = settingValue;
+	}
+	if (ValueString != NULL)
+	{
+	    *ValueString = NULL;
+	}
+    }
+}
+
+
 /**
  * gok_data_get_setting:
  * @NameAccessMethod: Name of the access method that contains the setting.
@@ -450,44 +491,37 @@ gboolean gok_data_get_setting (gchar* NameAccessMethod, gchar* NameSetting, gint
 		NameSetting = buffer;
 	}
 	
-	/* find a value for the given access method setting in our list of settings */
-	pSetting = m_pSettingFirst;
-	while (pSetting != NULL)
+	/* first look to see if we have a commandline override for the setting */
+	if (gok_main_get_scan_override () && !strcmp (gok_data_get_name_accessmethod (), NameAccessMethod) && 
+	    !strcmp (NameSetting, "movehighlighter") && gok_action_find_action (gok_main_get_scan_override (), FALSE))
 	{
-		/* check if this setting has the same access method name and setting name as given */
-		if ((strcmp (pSetting->NameAccessMethod, NameAccessMethod) == 0) &&
-			(strcmp (pSetting->Name, NameSetting) == 0))
+		gok_data_fill_setting_values (0, gok_main_get_scan_override (), Value, ValueString);
+		return TRUE;
+	}
+	else if (gok_main_get_select_override () && 
+		 !strcmp (gok_data_get_name_accessmethod (), NameAccessMethod) &&
+		 (!strcmp (NameSetting, "select") || !strcmp (NameSetting, "outputselected"))
+		 && gok_action_find_action (gok_main_get_select_override (), FALSE))
+	{
+		gok_data_fill_setting_values (0, gok_main_get_select_override (), Value, ValueString);
+		return TRUE;
+	}
+	else
+	{
+		/* find a value for the given access method setting in our list of settings */
+		pSetting = m_pSettingFirst;
+		while (pSetting != NULL)
 		{
-			/* set the given value to the new value */
-			if (pSetting->pValueString != NULL)
+			/* check if this setting has the same access method name and setting name as given */
+			if ((strcmp (pSetting->NameAccessMethod, NameAccessMethod) == 0) &&
+			    (strcmp (pSetting->Name, NameSetting) == 0))
 			{
-				if (ValueString != NULL)
-				{
-					*ValueString = pSetting->pValueString;
-				}
-				if (Value != NULL)
-				{
-					*Value = 0;
-				}
+				gok_data_fill_setting_values (pSetting->Value, pSetting->pValueString, Value, ValueString);
+				return TRUE;
 			}
-			else
-			{
-				if (Value != NULL)
-				{
-					*Value = pSetting->Value;
-				}
-				if (ValueString != NULL)
-				{
-					*ValueString = NULL;
-				}
-			}
-			
-			return TRUE;
+			pSetting = pSetting->pSettingNext;
 		}
-
-		pSetting = pSetting->pSettingNext;
 	}
-
 	return FALSE;
 }
 
@@ -700,7 +734,7 @@ gboolean gok_data_get_use_xkb_kbd ()
 void gok_data_set_use_xkb_kbd (gboolean val)
 {
 	m_bUseXkbKbd = val;
-	gok_gconf_set_int (gconf_client, GOK_GCONF_USE_XKB_KBD,
+	gok_gconf_set_bool (gconf_client, GOK_GCONF_USE_XKB_KBD,
 				m_bUseXkbKbd);
 	m_bSettingsChanged = TRUE;
 }
@@ -728,12 +762,15 @@ void gok_data_set_dock_type (GokDockType val)
 	switch (val) {
 	case GOK_DOCK_TOP:
 	  typestring = "top";
+	  gok_main_set_wm_dock (TRUE);
 	  break;
 	case GOK_DOCK_BOTTOM:
 	  typestring = "bottom";
+	  gok_main_set_wm_dock (TRUE);
 	  break;
 	default:
 	  typestring = "";
+	  gok_main_set_wm_dock (FALSE);
 	  break;
 	}
 	gok_gconf_set_string (gconf_client, GOK_GCONF_DOCK_TYPE,
@@ -855,7 +892,8 @@ void gok_data_set_keysize_priority (gboolean bFlag)
  *
  * This function does not write to GConf it only copies the string
  */
-void gok_data_set_name_accessmethod_internal (char* Name)
+static void 
+gok_data_set_name_accessmethod_internal (const char* Name)
 {
 	if (strcmp (Name, NameAccessMethod) == 0)
 	{
@@ -873,7 +911,7 @@ void gok_data_set_name_accessmethod_internal (char* Name)
  *
  * Records the new access method and stores it in GConf.
  */
-void gok_data_set_name_accessmethod (char* Name)
+void gok_data_set_name_accessmethod (const char* Name)
 {
 	if (gok_main_get_access_method_override() != NULL)
 	{
@@ -1044,9 +1082,16 @@ gboolean gok_data_get_expand ()
  */
 void gok_data_set_expand (gboolean bTrueFalse)
 {
+	GokKeyboard *kbd = gok_main_get_current_keyboard ();
 	m_bExpandWindow= bTrueFalse;
 	gok_gconf_set_bool (gconf_client, GOK_GCONF_EXPAND,
 			    m_bExpandWindow);
+	if (kbd)
+	{
+		gok_keyboard_display (kbd,
+				      kbd,
+				      gok_main_get_main_window (), TRUE);
+	}
 	m_bSettingsChanged = TRUE;
 }
 
@@ -1188,16 +1233,97 @@ gboolean gok_data_restore_setting (GokSetting* pSetting)
 gdouble
 gok_data_get_valuator_sensitivity (void)
 {
-	/* TODO: implement in gconf */
-	return 0.25;
+	if (gok_main_get_valuatorsensitivity_override() != 0) {
+		return gok_main_get_valuatorsensitivity_override();
+	}
+	return m_ValuatorSensitivity;
 }
 
 void 
 gok_data_set_valuator_sensitivity (gdouble multiplier)
 {
-	/* TODO: implement in gconf */
+	if (gok_main_get_valuatorsensitivity_override() != 0) {
+		return;
+	}
+	m_ValuatorSensitivity = multiplier;
+	gok_gconf_set_double (gconf_client, GOK_GCONF_VALUATOR_SENSITIVITY,
+			      m_ValuatorSensitivity);
+	m_bSettingsChanged = TRUE;
 }
 
+const gchar*
+gok_data_compose_keyboard_type_string (GokComposeType type)
+{
+    switch (type)
+    {
+	case GOK_COMPOSE_XKB:
+	    return "xkb";
+	case GOK_COMPOSE_ALPHA:
+	    return "alpha";
+	case GOK_COMPOSE_ALPHAFREQ:
+	    return "alpha-freq";
+	case GOK_COMPOSE_CUSTOM:
+	    return "custom";
+	case GOK_COMPOSE_DEFAULT:
+	default:
+	    return "default";
+    }
+}
+
+static const GokComposeType
+gok_data_compose_type_from_string (const gchar *s)
+{
+	if (!strcmp ("xkb", s)) return GOK_COMPOSE_XKB;
+	else if (!strcmp ("alpha", s)) return GOK_COMPOSE_ALPHA;
+	else if (!strcmp ("alpha-freq", s)) return GOK_COMPOSE_ALPHAFREQ;
+	else if (!strcmp ("custom", s)) return GOK_COMPOSE_CUSTOM;
+	else return GOK_COMPOSE_DEFAULT;
+}
+
+GokComposeType 
+gok_data_get_compose_keyboard_type (void)
+{
+    return compose_keyboard_type;
+}
+
+void 
+gok_data_set_compose_keyboard_type (GokComposeType type)
+{
+    compose_keyboard_type = type;
+    gok_gconf_set_string (gconf_client, GOK_GCONF_COMPOSE_KBD_TYPE,
+			  gok_data_compose_keyboard_type_string (type));
+    gok_data_set_use_xkb_kbd (type == GOK_COMPOSE_XKB);    
+}
+
+gchar *
+gok_data_get_custom_compose_filename (void)
+{
+	gchar *s;
+	return (gok_gconf_get_string (gconf_client, GOK_GCONF_CUSTOM_COMPOSE_FILENAME,
+				     &s) ? s : NULL);
+}
+
+void 
+gok_data_set_custom_compose_filename (const gchar *filename)
+{
+	gok_gconf_set_string (gconf_client, GOK_GCONF_CUSTOM_COMPOSE_FILENAME, filename);
+}
+
+gchar *
+gok_data_get_aux_keyboard_directory (void)
+{
+	gchar *s;
+	return (gok_gconf_get_string (gconf_client, GOK_GCONF_AUX_KEYBOARD_DIRECTORY,
+				     &s) ? s : NULL);
+}
+
+void 
+gok_data_set_aux_keyboard_directory (const gchar *dirname)
+{
+	gok_gconf_set_string (gconf_client, GOK_GCONF_AUX_KEYBOARD_DIRECTORY, dirname);
+}
+
+
 gint
 gok_data_get_repeat_rate (void)
 {
@@ -1215,10 +1341,10 @@ gok_data_set_repeat_rate (gint rate)
 /* 
  * gok_data_get_aux_dictionaries:
  *
- * Returns a constant string which specifies a semicolon-delimited
+ * Returns a string which specifies a semicolon-delimited
  * list of fully-qualified paths to word lists.
  */
-const gchar* 
+gchar* 
 gok_data_get_aux_dictionaries (void)
 {
 	return m_AuxDicts;
