@@ -112,13 +112,13 @@ gok_key_lookup_shorter_label (guchar *str)
 		xstr = "Vol\n-";
 	}
 	else if (!strcmp (str, "SunAudioMute")) {
-		xstr = "Mute";
+		xstr = _("Mute");
 	}
 	else if (!strcmp (str, "Pointer\nEnableKeys")) {
-		xstr = "Mouse\nKeys";
+		xstr = _("Mouse\nKeys");
 	}
 	else if (!strcmp (str, "ISO_Left\nTab")) {
-		xstr = "Left\nTab";
+		xstr = _("Left\nTab");
 	}
 	else if (!strcmp (str, "Sterling")) {
 		xstr = "£";
@@ -228,9 +228,76 @@ gok_key_label_from_keysym_string (const guchar *str)
 	if (!str) return g_strdup ("<nil>");
 	strl = strlen(str);
 
-	if (strncmp(str, "KP_", 3) == 0) {
-	        cp = g_strdup (str + 3);
+	if (!strcmp (str, "KP_Divide")) {
+		cp = _("Divide");
 	}
+	else if (!strcmp (str, "KP_Multiply")) {
+		cp = _("Multiply");
+	}
+	else if (!strcmp (str, "KP_Subtract")) {
+		cp = _("Subtract");
+	}
+	else if (!strcmp (str, "KP_Add")) {
+		cp = _("Addition");
+	}
+	else if (!strcmp (str, "KP_Prior") || !strcmp (str, "Prior")) {
+		cp = _("Prior");
+	}
+	else if (!strcmp (str, "KP_Next") || !strcmp (str, "Next")) {
+		cp = _("Next");
+	}
+	else if (!strcmp (str, "KP_Home")) {
+		cp = _("Home");
+	}
+	else if (!strcmp (str, "KP_End")) {
+		cp = _("End");
+	}
+	else if (!strcmp (str, "KP_Up")) {
+		cp = _("Up");
+	}
+	else if (!strcmp (str, "KP_Down")) {
+		cp = _("Down");
+	}
+	else if (!strcmp (str, "KP_Left")) {
+		cp = _("Left");
+	}
+	else if (!strcmp (str, "KP_Right")) {
+		cp = _("Right");
+	}
+	else if (!strcmp (str, "KP_Begin")) {
+		cp = _("Begin");
+	}
+	else if (!strcmp (str, "KP_Decimal")) {
+		cp = _("Decimal");
+	}
+	else if (!strcmp (str, "Meta_L") || !strcmp (str, "Meta_R")) {
+		cp = _("Meta");
+	}
+	else if (!strcmp (str, "Multi_key")) {
+		cp = _("Multi\nkey");
+	}
+	else if (!strcmp (str, "Eisu_toggle")) {
+		cp = _("Eisu\ntoggle");
+	}
+	else if (!strcmp (str, "Henkan_Mode")) {
+		cp = _("Henkan\nMode");
+	}
+	else if (!strcmp (str, "Muhenkan")) {
+		cp = _("Muhenkan");
+	}
+	else if (!strcmp (str, "Mode_switch")) {
+		cp = _("Mode\nswitch");
+	}
+/*      REMOVE COMMENTS POST-BRANCH: freeze applies currently */
+ 	else if (!strcmp (str, "Hiragana_Katakana")) {
+	        cp = "Hiragana\nKatakana";
+/*
+	        cp = _("Hiragana\nKatakana");
+ */
+	}
+	else if (strncmp(str, "KP_", 3) == 0) {
+ 	        cp = g_strdup (str + 3);
+ 	}
 	else if ((strl >= 2) && (str[strl-1] == 'L') && (str[strl-2] == '_')) {
 		cp = g_strndup (str, strl - 2);
 	}
@@ -240,7 +307,7 @@ gok_key_label_from_keysym_string (const guchar *str)
 	else {
 	        cp = g_strdup (str);
 	}
-	if (g_strrstr (cp, "_")) {
+	if ((strl > 1) && g_strrstr (cp, "_")) {
 		*g_strrstr (cp, "_") = '\n';
 	}
 	if (strlen (cp) > 4)
@@ -263,7 +330,7 @@ gok_key_get_numlock_mask (Display *display)
 }
 
 
-static int
+int
 gok_key_get_xkb_type_index (XkbDescPtr xkb, KeyCode keycode, guint group)
 {
 	int num_groups = XkbKeyNumGroups (xkb, keycode);
@@ -275,7 +342,7 @@ gok_key_get_xkb_type_index (XkbDescPtr xkb, KeyCode keycode, guint group)
 	return xkb->map->key_sym_map[keycode].kt_index[group];
 }
 
-static int
+int
 gok_key_level_for_type (Display *display, XkbDescRec *kbd,
 			int type, unsigned int *modmask)
 {
@@ -357,7 +424,7 @@ gok_key_label_from_keycode (KeyCode keycode, Display *display, guint level, guin
 {
   char buf[20];
   int extra_rtn, nbytes, state = 0;
-  KeySym keysym;
+  KeySym keysym = 0;
   XkbStateRec xkb_state;
   long ucs;
   gunichar unichar;
@@ -365,15 +432,18 @@ gok_key_label_from_keycode (KeyCode keycode, Display *display, guint level, guin
   keysym = XkbKeycodeToKeysym (display, keycode, group, level);
   if ((ucs = keysym2ucs (keysym)) <= 0) 
   {
+      gint reported_nbytes = XkbTranslateKeySym (display, &keysym, 0, buf, 19, &extra_rtn);
+      gchar *s = NULL;
       /* workaround; XSun gets nbytes wrong, so we truncate to the largest legal value */  
-      nbytes = MIN (XkbTranslateKeySym (display, &keysym, 0, buf, 19, &extra_rtn), 19);
+      nbytes = MIN (reported_nbytes, 19);
       buf[nbytes] = '\0';
-      return gok_key_label_from_keysym_string (XKeysymToString (keysym));
+      if (keysym) s = XKeysymToString (keysym);
+      return gok_key_label_from_keysym_string (s);
   }
   else
   {
-      unichar = (gunichar) ucs;
       gchar cbuf[10];
+      unichar = (gunichar) ucs;
       cbuf[g_unichar_to_utf8 (unichar, cbuf)] = '\0';
 
       return gok_key_label_from_keysym_string (cbuf);
@@ -518,7 +588,7 @@ gok_key_from_xkb_key (GokKey *prevKey, GokKeyboard *pKeyboard, Display *display,
 	  is_modifier = TRUE;
 	  gok_modifier_add (pKey->ModifierName);
   }
-  if (is_modifier) { /* FIXME */
+  if (is_modifier) {
     pKey->Type = KEYTYPE_MODIFIER;
     pKey->pImage = gok_keyimage_new (pKey, NULL);
     pKey->pImage->type = IMAGE_TYPE_INDICATOR;
@@ -632,6 +702,7 @@ GokKey* gok_key_new (GokKey* pKeyPrevious, GokKey* pKeyNext, GokKeyboard* pKeybo
 	pgok_key_new->State = GTK_STATE_NORMAL;
 	pgok_key_new->StateWhenNotFlashed = GTK_STATE_NORMAL;
 	pgok_key_new->pGeneral = NULL;
+	pgok_key_new->action_ndx = 0;
 	pgok_key_new->is_repeatable = FALSE;
 	
 	return pgok_key_new;
@@ -734,13 +805,13 @@ void gok_key_delete (GokKey* pKey, GokKeyboard* pKeyboard, gboolean bDeleteButto
 		gok_spy_accessible_unref (pKey->accessible_node->paccessible);
 	} 
 
-	g_free (pKey);
-
 	gok_feedback_drop_refs (pKey);
 	gok_scanner_drop_refs (pKey);
 	gok_repeat_drop_refs (pKey);
 
 	if (pKey->Type != KEYTYPE_WINDOW) g_free (pKey->pGeneral);
+
+	g_free (pKey);
 }
 
 /**
@@ -801,6 +872,11 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 		pKey->Type = KEYTYPE_BRANCHBACK;
 		pKey->Style = KEYSTYLE_BRANCHBACK;
 	}
+	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *) "branchCompose") == 0)
+	{
+		pKey->Type = KEYTYPE_BRANCHCOMPOSE;
+		pKey->Style = KEYSTYLE_BRANCH;
+	}
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"branchAlphabet") == 0)
 	{
 		pKey->Type = KEYTYPE_BRANCHALPHABET;
@@ -823,7 +899,7 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 	}
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"branchEditText") == 0)
 	{
-		pKey->Type = KEYTYPE_BRANCHCOMPOSE;
+		pKey->Type = KEYTYPE_BRANCHEDIT;
 		pKey->Style = KEYSTYLE_GENERALDYNAMIC;
 	}
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"branchSettings") == 0)
@@ -863,7 +939,7 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 		{
 			* (GokKeyboardDirection *) pKey->pGeneral = 
 				gok_keyboard_parse_direction (pStringSubAttributeValue);
-			g_free (pStringSubAttributeValue);
+			xmlFree (pStringSubAttributeValue);
 		}
 	}
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"move-resize") == 0)
@@ -877,7 +953,7 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 		{
 			* (GokKeyboardDirection *) pKey->pGeneral = 
 				gok_keyboard_parse_direction (pStringSubAttributeValue);
-			g_free (pStringSubAttributeValue);
+			xmlFree (pStringSubAttributeValue);
 		}
 		pKey->is_repeatable = TRUE;
 	}
@@ -904,6 +980,21 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 		pKey->Style = KEYSTYLE_NORMAL;
 		gok_compose_key_init (pKey, pNode);
 	}
+	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"valuator") == 0)
+	{
+		pKey->Type = KEYTYPE_VALUATOR;
+		pKey->Style = KEYSTYLE_NORMAL;
+		pKey->is_repeatable = TRUE;
+		pKey->pGeneral = g_new0 (GokKeyboardValueOp, 1);
+		pStringSubAttributeValue = 
+			xmlGetProp (pNode, (const xmlChar *) "command");
+		if (pStringSubAttributeValue != NULL) 
+		{
+			* (GokKeyboardValueOp *) pKey->pGeneral = 
+				gok_keyboard_parse_value_op (pStringSubAttributeValue);
+			xmlFree (pStringSubAttributeValue);
+		}
+	}
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"select") == 0)
 	{
 		pKey->Type = KEYTYPE_SELECT;
@@ -1163,7 +1254,11 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 * gok_keylabel_new
 * @pKey: Pointer to the key that gets the new label.
 * @pLabelText: Text string for this label.
-* @pModifierText: Modifier text string (e.g. "shift" or "ctrl").
+* @level: the level (see XKB spec) for which this label is valid.
+* @group: the group (see XKB spec) for which this label is valid.
+* @vmods: a delimited list of virtual modifier names which must be matched 
+*         in order for this label to be valid, or NULL if no virtual 
+*         modifiers are relevant to this label.
 *
 * Allocates memory for a new key label and initializes the GokKeyLabel structure.
 * Returns a pointer to the new key label, NULL if it can't be created.
@@ -1523,31 +1618,45 @@ gchar* gok_key_get_label (GokKey* pKey)
 	gchar buffer[150];
 	GokKeyLabel* pKeyLabel;
 	gboolean bFoundLabel;
-	int  type, keycode;
+	int  type = XkbAlphabeticIndex, keycode;
 	guint level;
 	guint group = gok_key_get_effective_group ();
 	guint mods = gok_spy_get_modmask ();
-	XkbDescRec *xkb;
-
-	xkb = gok_keyboard_get_xkb_desc (),
-	keycode = gok_output_get_keycode (pKey->pOutput);
-	type = gok_key_get_xkb_type_index (xkb, keycode, group);
+	XkbDescRec *xkb = gok_keyboard_get_xkb_desc ();
+	GokOutputType output_type = OUTPUT_INVALID;
 
 #ifdef GOK_DEBUG
 	char *typename = XGetAtomName (gok_keyboard_get_display (), xkb->map->types[type].name);
-	fprintf (stderr, "keycode %d, group %d, type=%d [%s]\n", keycode, group, type, typename);
 #endif
-	level = gok_key_level_for_type (gok_keyboard_get_display (), 
-					xkb,
-					type, &mods);
 
+	if (pKey->pOutput) output_type = pKey->pOutput->Type;
+	switch (output_type)
+	{
+	    case OUTPUT_KEYCODE:
+	    case OUTPUT_KEYSYM:
+		keycode = gok_output_get_keycode (pKey->pOutput);
+		if (keycode > 0)
+		    type = gok_key_get_xkb_type_index (xkb, keycode, group);
 #ifdef GOK_DEBUG
-	if (pKey->pLabel) fprintf (stderr, "Key %s: type %d, level %d [mods %x, preserved %x]\n", 
-				pKey->pLabel->Text,
-				type, level, gok_spy_get_modmask (), mods);
+		fprintf (stderr, "keycode %d, group %d, type=%d [%s]\n", keycode, group, type, typename);
 #endif
+		level = gok_key_level_for_type (gok_keyboard_get_display (), 
+						xkb,
+						type, &mods);
+#ifdef GOK_DEBUG
+		if (pKey->pLabel) fprintf (stderr, "Key %s: type %d, level %d [mods %x, preserved %x]\n", 
+					   pKey->pLabel->Text,
+					   type, level, gok_spy_get_modmask (), mods);
+#endif
+		break;
+	    case OUTPUT_KEYSTRING:
+	    default:
+		level = gok_key_level_for_type (gok_keyboard_get_display (), xkb, XkbAlphabeticIndex, &mods);
+		break;
+	}
+		
 	g_assert (pKey != NULL);
-	
+		
 	pTextReturned = m_TextEmpty;
 	
 	/* get the 'normal' label */
@@ -1577,7 +1686,8 @@ gchar* gok_key_get_label (GokKey* pKey)
 	{
 		if (pKeyLabel->level == level && pKeyLabel->group == group) 
 		{
-			return pKeyLabel->Text;
+			pTextReturned = pKeyLabel->Text;
+			break;
 		}
 		pKeyLabel = pKeyLabel->pLabelNext;
 	}
@@ -1753,6 +1863,7 @@ void gok_key_set_button_name (GokKey* pKey)
 		strcpy (styleText, "StyleTextNormal");
 		break;
 					
+	case KEYSTYLE_ADDWORD:
 	case KEYSTYLE_HELP:
 		strcpy (styleButton, "StyleButtonHelp");
 		strcpy (styleText, "StyleTextHelp");
@@ -1858,7 +1969,7 @@ int gok_key_calculate_font_size (GokKey* pKey, gboolean bWidth, gboolean bHeight
 	if (bWidth == TRUE)
 	{
 		/* get the size of a key cell*/
-	        widthCell = gok_data_get_key_width () * 
+	        widthCell = gok_keyboard_get_cell_width (gok_main_get_current_keyboard ()) * 
 		        (pKey->Right - pKey->Left) - 2 * 
 		        gok_key_get_default_border_width (pKey) - 1;
 	
@@ -2139,6 +2250,7 @@ int gok_key_get_label_lengthpercell (GokKey* pKey)
 		return 0;
 	
 	numCells = pKey->Right - pKey->Left;
+	if (numCells <= 0) return 10;
 	return gok_key_get_label_length (pKey) / numCells;
 }
 
@@ -2355,7 +2467,12 @@ gok_key_isRepeatable(GokKey* pKey)
 	return returncode;
 }
 
-
+/* 
+ * the reason for this static: GokOutput is not a GObject, 
+ * so we have no ref counting ability.
+ * therefore we only delete a wordcomplete output when creating a new one. 
+ * It would be better to do proper ref counting.
+ */
 static GokOutput *last_wordcomplete_output = NULL;
 
 /**
@@ -2387,25 +2504,28 @@ gok_key_wordcomplete_output (GokKey *pKey, GokWordComplete *complete)
 	{
 		return NULL;
 	}
-	
+
+	if (last_wordcomplete_output) {
+	    gok_output_delete_all (last_wordcomplete_output);
+	    last_wordcomplete_output = NULL;
+	}
+
 	if (word_part && (g_utf8_strlen (word_part, -1) < g_utf8_strlen (LabelText, -1)))
 	{
-		gchar *cp;
-                /* 
-		 * note that this won't match if the decompositions don't match. Should we
-		 * decompose to canonical first ? 
-		 */ 
-		cp = strstr (LabelText, word_part); 
+		gchar *cp, *word_part_canonical;
+
+		word_part_canonical = g_utf8_normalize (word_part, -1, G_NORMALIZE_DEFAULT);
+		cp = strstr (LabelText, word_part_canonical); 
 		/* sanity check: is our word_part found in our label? */
 		if (cp) {
-			gint offset = strlen (word_part); /* in bytes, not chars */
-			if (last_wordcomplete_output) gok_output_delete_all (last_wordcomplete_output);
+			gint offset = strlen (word_part_canonical); /* in bytes, not chars */
 			last_wordcomplete_output = gok_output_new (OUTPUT_KEYSTRING, cp + offset, SPI_KEY_STRING);
 		}
 		else {
 			/* if not then we are doing a 'replace', handle that via EditableText. */
 			g_warning ("Sorry, string replacement is not yet implemented.");
 		}
+		g_free (word_part_canonical);
 	}
 
 	return last_wordcomplete_output;
