@@ -310,6 +310,11 @@ gok_pointer_clip_to_window (gint *xp, gint *yp, GdkWindow *window, gint slop)
 /**
 * gok_scanner_initialize:
 * @directory: The name of the directory to read the access method files from.
+* @accessmethod: If non-NULL, overrides access method name in gconf configuration data.
+* @selectaction: If non-NULL, overrides action associated with 'select' for 
+*                the current access method in gconf configuration data.
+* @scanaction: If non-NULL, overrides access method associated with 'select' for 
+*                the current access method in gconf configuration data.
 *
 * Reads all the access methods from the given directory
 * and gets them ready to go.
@@ -317,7 +322,8 @@ gok_pointer_clip_to_window (gint *xp, gint *yp, GdkWindow *window, gint slop)
 * returns: TRUE if the access methods were initialized, FALSE if not. Don't use the 
 * access methods if this fails.
 **/
-gboolean gok_scanner_initialize (const gchar *directory)
+gboolean gok_scanner_initialize (const gchar *directory, const gchar *accessmethod, 
+				 const gchar *selectaction, const gchar *scanaction)
 {
 	DIR* pDirectoryAccessMethods;
 	struct dirent* pEntry;
@@ -376,7 +382,22 @@ gboolean gok_scanner_initialize (const gchar *directory)
 	/* get the rates for all the access methods */
 	gok_scanner_update_rates ();
 
-	/* set the current access method */
+	if (accessmethod) /* we've specified the name in the args list */
+	{
+	    m_pAccessMethodCurrent = m_pAccessMethodFirst;
+	    while (m_pAccessMethodCurrent)
+	    { 
+		if (!strcmp (m_pAccessMethodCurrent->Name, accessmethod)) 
+		{
+		    gok_data_set_name_accessmethod (accessmethod);
+		    gok_scanner_reset_access_method ();
+		    g_message ("using access method %s", accessmethod);
+		    return TRUE;
+		}
+		m_pAccessMethodCurrent = m_pAccessMethodCurrent->pAccessMethodNext;
+	    }
+	}
+	/* set the current access method from gconf value (note fall-through if match above failed) */
 	return gok_scanner_change_method (gok_data_get_name_accessmethod());
 }
 
@@ -527,7 +548,7 @@ gboolean gok_scanner_read_access_method (gchar* Filename)
 		}
 	}
 		
-	strcpy (pAccessMethod->DisplayName, (gchar *)pStringAttributeValue);
+	strcpy (pAccessMethod->DisplayName, _((gchar *)pStringAttributeValue));
 
 	/* read the UI for the access method */
 	gok_scanner_read_description (pDoc, pNodeAccessmethod->xmlChildrenNode, pAccessMethod);
@@ -972,14 +993,14 @@ gboolean gok_scanner_read_description (xmlDoc* pDoc, xmlNode* pNode, GokAccessMe
 	if (pNode = gok_scanner_find_node (pNode, (const xmlChar *) "description", gok_scanner_get_lang (),
 					   NULL)) {
 		pStringDescription = xmlNodeListGetString (pDoc, pNode->xmlChildrenNode, 1);
-		strncpy (pAccessMethod->Description, (gchar *)pStringDescription, MAX_DESCRIPTION_TEXT);
+		g_strlcpy (pAccessMethod->Description, _((gchar *)pStringDescription), MAX_DESCRIPTION_TEXT);
 		gok_log ("read access method description: [%s]", pStringDescription);
-		g_free (pStringDescription);
+		xmlFree (pStringDescription);
 		gok_log_leave();
 		return TRUE;
 	}
 	else {
-		strncpy (pAccessMethod->Description, (gchar *)"error reading description", MAX_DESCRIPTION_TEXT);
+		g_strlcpy (pAccessMethod->Description, (gchar*)_("error reading description"), MAX_DESCRIPTION_TEXT);
 		gok_log_leave();
 		return FALSE;
 	}
@@ -2316,7 +2337,7 @@ void gok_scanner_input_motion (gint *motion_data, gint n_axes)
 		x -= o_x;
 		y -= o_y;
 	}
-	pKey = gok_keyboard_find_key_at (pKeyboard, x, y);
+	pKey = gok_keyboard_find_key_at (pKeyboard, x, y, m_pContainingKey);
 	gok_keyboard_unpaint_pointer (pKeyboard, pWindow);
 	if (m_pContainingKey != pKey) {
 		gok_scanner_on_key_leave (m_pContainingKey);
@@ -2672,51 +2693,19 @@ void gok_scanner_perform_effects (GokScannerEffect* pEffect)
 				break;
 
 			case CALL_CHUNKER_WRAP_TOBOTTOM:
-				if (gok_data_get_setting (pAccessMethod->Name, pEffect->pName, 
-											&rate, NULL) == TRUE)
-				{
-					gok_chunker_wraptobottom (rate); 
-				}
-				else
-				{
-					gok_log_x ("Can't get CHUNKER_WRAP_TOBOTTOM name setting: %s!\n", pEffect->pName);
-				}
+				gok_chunker_wraptobottom (rate); 
 				break;
 
 			case CALL_CHUNKER_WRAP_TOLEFT:
-				if (gok_data_get_setting (pAccessMethod->Name, pEffect->pName, 
-											&rate, NULL) == TRUE)
-				{
-					gok_chunker_wraptoleft (rate); 
-				}
-				else
-				{
-					gok_log_x ("Can't get CHUNKER_WRAP_TOLEFT name setting: %s!\n", pEffect->pName);
-				}
+				gok_chunker_wraptoleft (rate); 
 				break;
 
 			case CALL_CHUNKER_WRAP_TORIGHT:
-				if (gok_data_get_setting (pAccessMethod->Name, pEffect->pName, 
-											&rate, NULL) == TRUE)
-				{
-					gok_chunker_wraptoright (rate); 
-				}
-				else
-				{
-					gok_log_x ("Can't get CHUNKER_WRAP_TORIGHT name setting: %s!\n", pEffect->pName);
-				}
+				gok_chunker_wraptoright (rate);
 				break;
 
 			case CALL_CHUNKER_WRAP_TOTOP:
-				if (gok_data_get_setting (pAccessMethod->Name, pEffect->pName, 
-											&rate, NULL) == TRUE)
-				{
-					gok_chunker_wraptotop (rate); 
-				}
-				else
-				{
-					gok_log_x ("Can't get CHUNKER_WRAP_TOTOP name setting: %s!\n", pEffect->pName);
-				}
+				gok_chunker_wraptotop (rate); 
 				break;
 
 			case CALL_CHUNKER_MOVE_LEFTRIGHT:
@@ -2778,7 +2767,6 @@ gok_log_x ("chunker move topbottom\n");
 				break;
 
 			case CALL_CHUNKER_IF_KEY_SELECTED:
-gok_log_x ("chunker  if key selected\n");
 				EffectCodeReturned = gok_chunker_if_key_selected(); 
 				break;
 
@@ -3353,10 +3341,14 @@ gok_scanner_current_state_uses_corepointer (void)
 		gok_gconf_get_string (gconf_client_get_default (), 
 				      GOK_GCONF_INPUT_DEVICE, 
 				      &input_device_name);
+
 		if (input_device_name == NULL)
 			retval = TRUE;
 		else
 			g_free (input_device_name);
+		
+		if (retval && (gok_main_get_inputdevice_name () != NULL))
+			retval = FALSE;
 	}
 
 	return retval;
